package goaml.lang.parameter;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

import goaml.lang.StateChart;
import goaml.lang.entity.Entity;
import goaml.lang.state.AbstractState;

public final class Reference {

	private final String NAME;
	private final AbstractState STATE;

	private Reference(final String name, final AbstractState state) {
		this.NAME = name;
		this.STATE = state;
	}

	// From ONE "Reference" tag, create and return a list of corresponding instances of Reference
	public static final List<Reference> parse(final Node node, final StateChart stateChart) {
		if (node == null) {
			throw new RuntimeException("Node is null");
		}
		if (node.getNodeName() != "Reference") {
			throw new RuntimeException("NodeName \"Reference\" was expected, got : " + node.getNodeName());
		}

		List<Reference> references = new ArrayList<Reference>();

		String name;
		List<Entity> entities;
		String stateName;

		NamedNodeMap nodeMap = node.getAttributes();

		// Required attributes
		if (nodeMap.getNamedItem("name") != null) {
			name = nodeMap.getNamedItem("name").getTextContent();
		} else {
			throw new RuntimeException("\"name\" attribute missed");
		}

		if (nodeMap.getNamedItem("entity") != null) {
			entities = stateChart.getEntitiesByName(nodeMap.getNamedItem("entity").getTextContent());
		} else {
			throw new RuntimeException("\"entity\" attribute missed");
		}

		if (nodeMap.getNamedItem("state") != null) {
			stateName = nodeMap.getNamedItem("state").getTextContent();
		} else {
			throw new RuntimeException("\"state\" attribute missed");
		}

		for (Entity entity : entities) {
			for (AbstractState state : entity.getStatesByName(stateName)) {
				references.add(new Reference(name, state));
			}
		}

		return references;
	}

	public final String getName() {
		return this.NAME;
	}

	public final AbstractState getState() {
		return this.STATE;
	}

}
