package goaml.lang.transition;

import java.util.ArrayList;
import java.util.List;

import goaml.lang.StateChart;
import goaml.lang.parameter.Constant;
import goaml.lang.parameter.Reference;
import goaml.lang.state.AbstractState;
import goaml.lang.state.State;
import goaml.lang.util.DelayedUpdate;

public final class DeathTReg extends AbstractTransition {

	private Constant deathRate;
	private Constant k2;
	private List<AbstractState> il2Producing;

	public DeathTReg() {
	}

	private DeathTReg(final State source, final State target, final Constant deathRate, final Constant k2,
			final List<AbstractState> il2Producing) {
		super(source, target);
		this.deathRate = deathRate;
		this.k2 = k2;
		this.il2Producing = il2Producing;
	}

	@Override
	protected void initialize(StateChart stateChart, State source, State target, List<Constant> constants,
			List<Reference> references) {

		// Initialize constants
		deathRate = Constant.getConstant("deathRate", constants, source, target);
		k2 = Constant.getConstant("k2", constants, source, target);

		il2Producing = new ArrayList<AbstractState>();
		for (Reference reference : references) {
			if (reference.getName().equals("il2")) {
				il2Producing.add(reference.getState());
			}
		}

		// Check if a constant or a reference hasn't been initialized
		if (deathRate == null || k2 == null || il2Producing.size() == 0) {
			return;
		}

		stateChart.addTransition(new DeathTReg(source, target, deathRate, k2, il2Producing));

	}

	@Override
	public void compute(final StateChart stateChart) {
		// Compute the rate
		double il2Value = 0.0;
		for (AbstractState state : this.il2Producing) {
			il2Value += state.getValue();
		}
		double rate = deathRate.getValue() * k2.getValue() * source.getValue() / (k2.getValue() + il2Value);

		// Create DelayedUpdates in order to apply computed value on the states when all transitions
		// and processes have compute
		stateChart.addDelayedUpdate(new DelayedUpdate(this.source, DelayedUpdate.Action.SUBSTRACT, rate));
		stateChart.addDelayedUpdate(new DelayedUpdate(this.target, DelayedUpdate.Action.ADD, rate));
	}

}