package goaml.lang.transition;

import java.util.List;

import goaml.lang.StateChart;
import goaml.lang.parameter.Constant;
import goaml.lang.parameter.Reference;
import goaml.lang.state.State;
import goaml.lang.util.DelayedUpdate;

public final class IL2Prolif extends AbstractTransition {

	private Constant proliferationRate;
	private Reference il2;

	public IL2Prolif() {
	}

	private IL2Prolif(final State source, final State target, final Constant proliferationRate, final Reference il2) {
		super(source, target);
		this.proliferationRate = proliferationRate;
		this.il2 = il2;
	}

	@Override
	protected void initialize(StateChart stateChart, State source, State target, List<Constant> constants,
			List<Reference> references) {

		// Initialize constants
		proliferationRate = Constant.getConstant("proliferationRate", constants, source, target);

		// Check if all attributes have been initialized
		if (proliferationRate == null) {
			return;
		}

		for (Reference reference : references) {
			if (reference.getName().equals("il2")) {
				il2 = reference;
			}

			// Check if all attributes have been initialized
			if (proliferationRate == null || il2 == null) {
				return;
			}

			stateChart
					.addTransition(new IL2Prolif(source, (State) source.getFollowingCohort(), proliferationRate, il2));
		}
	}

	@Override
	public final void compute(StateChart stateChart) {
		// Compute the rate
		double rate = this.proliferationRate.getValue() * this.il2.getState().getValue() * this.source.getValue();

		// Create DelayedUpdates in order to apply computed value on the states when all transitions
		// and processes have compute
		stateChart.addDelayedUpdate(new DelayedUpdate(this.source, DelayedUpdate.Action.SUBSTRACT, rate));
		stateChart.addDelayedUpdate(new DelayedUpdate(this.target, DelayedUpdate.Action.ADD, 2 * rate));
	}

}
