package goaml.lang.util;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

public class CSVWriter {

	private enum Type {
		SIMULATION;
	}

	public static final Type SIMULATION = Type.SIMULATION;

	private final char DEFAULT_SEPARATOR = ',';
	private final String EXTENSION = ".csv";

	private FileWriter writer;

	public CSVWriter(final Type type, final String stateChartName) throws IOException {
		this.writer = create(type, stateChartName);
	}

	// Create a new CSV file
	private FileWriter create(final Type type, final String stateChartName) throws IOException {
		Date today = new Date();
		SimpleDateFormat dateFormater = new SimpleDateFormat("yyyy-MM-dd HH-mm-ss");
		String dateToString = dateFormater.format(today);

		boolean exists = false;
		File file;
		StringBuilder path;
		int num = 1;
		do {
			path = new StringBuilder();
			path.append(type.toString().toLowerCase());
			path.append("-");
			path.append(stateChartName);
			path.append("-");
			path.append(dateToString);
			path.append(" -- ");
			path.append(num);
			path.append(EXTENSION);

			file = new File(path.toString());
			if (file.isFile()) {
				exists = true;
			} else {
				exists = false;
			}

			num++;

		} while (exists); // while there is already a file with name equals to "path"

		return new FileWriter(file);
	}

	public void write(final List<String> datas) throws IOException {
		boolean first = true;

		StringBuilder sb = new StringBuilder();
		for (String data : datas) {
			if (!first) {
				// Append separator before writing the string "data"
				// Only done when it isn't the first string of "datas"
				sb.append(DEFAULT_SEPARATOR);
			} else {
				first = false;
			}

			sb.append(data);
		}
		sb.append("\n");

		writer.append(sb);
	}

	public void flush() throws IOException {
		this.writer.flush();
	}

	public void close() throws IOException {
		this.writer.close();
	}

}
